##############################################################################
#
# Copyright (c) 2002 Ingeniweb SARL
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
# Zope imports
from AccessControl import ClassSecurityInfo
from ComputedAttribute import ComputedAttribute

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

# PloneArticle Permissions
from Products.PloneArticle import Permissions

class PloneArticleFunctions:
    """ a place to hold all the cool PloneArticle Functionality """
 

    security = ClassSecurityInfo()



    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Copy references from source to dest. We must keep attachment references."""
        
        is_copy = getattr(item, '_v_is_cp', None)
        at_tool = getToolByName(self, 'archetype_tool')
        
        # Get source and destination objects
        src_UID = self.UID()
        
        DocumentContent.manage_afterAdd(self, item, container)
        
        src_obj = at_tool.getObject(src_UID)
        dst_UID = self.UID()
        dst_obj = self
        
        if is_copy and src_obj is not None and dst_UID != src_UID:
            # Copy references from source Article
            # Compute new references for contents in Article
            # Avoid from renaming : dst_UID != src_UID
            for article_content in self.article_contents:
                for ref in article_content.getContentReferences(src_obj):
                    if hasattr(ref, article_content._content_brain_id):
                        brain = getattr(ref, article_content._content_brain_id)
                        content_uid = ref.targetUID
                        content_title = brain.Title()
                        content_description = brain.Description()
                        
                        if article_content.isContentContained(src_obj, content_uid):
                            # Get content in new article
                            content_obj = at_tool.getObject(content_uid)
                            
                            # Get id and get new content with new article with this ID
                            content_id = content_obj.getId()
                            
                            new_obj = getattr(dst_obj, content_id)
                            content_uid = new_obj.UID()
                        
                        article_content.addContentFromUID(dst_obj, content_uid)
                        article_content.editContent(dst_obj, content_uid, title=content_title, description=content_description)

    
    security.declareProtected(CMFCorePermissions.View, 'Description')
    def Description(self, **kwargs):
        """We have to override Description here to handle arbitrary
        arguments since PortalFolder defines it."""
        return self.getField('description').get(self, **kwargs)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setDescription')
    def setDescription(self, value, **kwargs):
        """We have to override setDescription here to handle arbitrary
        arguments since PortalFolder defines it."""
        self.getField('description').set(self, value, **kwargs)
    
    security.declareProtected(Permissions.EditPloneArticle, 'setText')
    def setText(self, value, **kwargs):
        """Body text mutator
        """
        allowable_content_types = getPloneArticleAllowableContentTypes(self)
        mimetype = kwargs.get('mimetype', None)
        
        if mimetype is None:
            # Select the good mimetype
            if len(allowable_content_types) == 1:
                kwargs['mimetype'] = allowable_content_types[0]
            elif len(allowable_content_types) == 0:
                kwargs['mimetype'] = 'text/html'
            
        # Set text field
        DocumentContent.setText(self, value, **kwargs)
    
    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        """Returns a concatination of all searchable text
        """
        
        attach_index_values = []
        
        for attach in self.getAttachmentBrains():
            # Append title
            attach_index_values.append(attach.Title())
            
            # Check if attach use AttachmentField
            attach_obj = attach.getObject()
            attach_field = attach_obj.getField('file')
            
            if hasattr(attach_field, 'isIndexed') and attach_field.isIndexed(attach_obj):
                # Append attach index value
                attach_index_values.append(attach_field.getIndexableValue(attach_obj))
            
        ret = "%s %s %s" % (
            DocumentContent.SearchableText(self),
            ' '.join(attach_index_values),
            ' '.join([x.Title() for x in self.getImageBrains()]) 
            )
            
        return ret


    #                                                                                   #
    #                               Tool indirections                                   #
    # We use those methods to allow people overloading their behaviour in their own     #
    # article subclasses.                                                               #

    security.declarePublic("getAllowedLinkTypes")
    def getAllowedLinkTypes(self):
        """
        Get allowed link types
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_link_types

    security.declarePublic("getAllowedLinkType")
    def getAllowedLinkType(self):
        """
        Get allowed link type
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_link_type

    security.declarePublic("getAllowedImageTypes")
    def getAllowedImageTypes(self):
        """
        Get allowed image types
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_image_types

    security.declarePublic("getAllowedImageType")
    def getAllowedImageType(self):
        """
        Get allowed image type
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_image_type

    security.declarePublic("getAllowedAttachmentTypes")
    def getAllowedAttachmentTypes(self):
        """
        Get allowed attachment types
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_attachment_types

    security.declarePublic("getAllowedAttachmentType")
    def getAllowedAttachmentType(self):
        """
        Get allowed attachment type
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_attachment_type

    security.declarePublic("getAllowedConferencepaperTypes")
    def getAllowedConferencepaperTypes(self):
        """
        Get allowed attachment types
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_attachment_types

    security.declarePublic("getAllowedConferencepaperType")
    def getAllowedConferencepaperType(self):
        """
        Get allowed attachment type
        """
        tool = getToolByName(self, 'portal_article')
        return tool.allowed_attachment_type

