#
# Test Meeting initialisation and set-up
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))
from Globals import package_home
from Products.Meeting.tests import meetingtc

PACKAGE_HOME = package_home(globals())

from common import * #borrowed from plonearticle
class TestFile:
    __allow_access_to_unprotected_subobjects__ = 1
    filename = 'afile.fil'
    def seek(*args): pass
    def tell(*args): return 0
    def read(*args): return 'file_contents'

class TestInstallation(meetingtc.MeetingTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        self.css        = self.portal.portal_css
        self.js        = self.portal.portal_javascripts
        self.kupu       = self.portal.kupu_library_tool
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.factory    = self.portal.portal_factory
        self.workflow   = self.portal.portal_workflow
        self.properties = self.portal.portal_properties

        self.metaTypes = ('Meeting',)

    def testSkinLayersInstalled(self):
        self.failUnless('Meeting' in self.skins.objectIds())
        
    def testTypesInstalled(self):
        for t in self.metaTypes:
            self.failUnless(t in self.types.objectIds())

    def testPortalFactorySetup(self):
        self.failUnless('Meeting' in self.factory.getFactoryTypes())

    def testCssInstalled(self):
        self.failUnless('meeting.css' in self.css.getResourceIds())

    def testJavascriptInstalled(self):
        self.failUnless('nifty.js' in self.js.getResourceIds())


    def testDefaultPageType(self):
        default_page_types = self.properties.site_properties.getProperty('default_page_types')
        self.failUnless('Meeting' in default_page_types)
    
    def testParentMetaTypesNotToQuery(self):
        parentMetaTypesNotToQuery = self.properties.navtree_properties.getProperty('parentMetaTypesNotToQuery')
        self.failUnless('Meeting' in parentMetaTypesNotToQuery)

    
    def testKupuResources(self):
        linkable = self.kupu.getPortalTypesForResourceType('linkable')
        mediaobject = self.kupu.getPortalTypesForResourceType('mediaobject')
        self.failUnless('Meeting' in linkable)

class TestContentCreation(meetingtc.MeetingTestCase):
    """Ensure content types can be created and edited"""

    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.folder.invokeFactory('Meeting', 'meeting1')
        self.meeting1 = getattr(self.folder, 'meeting1')

    def testEditMeeting(self):
        #import pdb; pdb.set_trace()
        self.meeting1.setTitle('A title')
        self.meeting1.setDescription('A description')
        self.meeting1.setText('<p>Body text</p>')

        self.assertEqual(self.meeting1.Title(), 'A title')
        self.assertEqual(self.meeting1.Description(), 'A description')
        self.assertEqual(self.meeting1.getText(), '<p>Body text</p>')



    def testMeetingSplitter(self):
        """
          when given a string in format 'title||description' returns a list in the form [title,description]
        """
        meetinglist = self.meeting1.meeting_splitter('title||description') 
        self.assertEqual(meetinglist[0],'title')
        self.assertEqual(meetinglist[1],'description')
    
    def testAddPresentation(self):
        #test_file = self.loadFile('test.sxw')
        self.failUnless(self.meeting1.addPresentation(
                title='title',
                description='description',
                file='test_file',
                ))

    def loadFile(name, size=0):
        """Load dummyfile from testing directory
        """
        path = os.path.join(PACKAGE_HOME, 'input', name)
        fd = open(path, 'rb')
        data = fd.read()
        fd.close()
        return data
    
    def createEmptyMeeting(self, container, content_id = 'meeting'):
        # return empty meeting
        container.invokeFactory(type_name='Meeting', id=content_id)
        self.failUnless(content_id in container.objectIds())
        meeting = getattr(container, content_id)
        self.assertEqual(meeting.title, '')
        self.assertEqual(meeting.getId(), content_id)
        self.assertEqual(len(meeting.getWorkingdocs()), 0)
        self.assertEqual(len(meeting.getImages()), 0)
        return meeting

    def createExternalFile(self, container=None, content_id='external_file'):
        container = container or self.member_folder
        container.invokeFactory(type_name='ATFile', id=content_id)
        self.failUnless(content_id in container.objectIds())
        return getattr(container, content_id)


    def addWorkingdoc(self, container=None, title = 'attachment_title', description = 'attachment_description'):
        """
        """
        container = self.meeting1
        self.meeting1.addWorkingdoc(self, title, description, file=TestFile())
    
    def addInformationdoc(self, container=None, title = 'attachment_title', description = 'attachment_description'):
        """
        """
        container = self.meeting1
        self.meeting1.addInformationdoc(self, title, description, file=TestFile())

    def addReferencedoc(self, container=None, title = 'attachment_title', description = 'attachment_description'):
        """
        """
        container = self.meeting1
        self.meeting1.addReferencedoc(self, title, description, file=TestFile())

    def addConferencepaper(self, container=None, title = 'attachment_title', description = 'attachment_description'):
        """
        """
        container = self.meeting1
        #container or self.testCreateArticle()
        self.meeting1.addConferencepaper(self, title, description, file=TestFile())



    def testAddWorkingdoc(self, ):
        """
        """
        self.addWorkingdoc()
        contents = self.meeting1.meeting_attachments
        attachments = contents[1].getContents(self.meeting1) #workingdoc has an index of 1 
        #the meeting1 has only one attachment
        self.assertEqual(1, len(attachments))

        #the attachment added is an File
        self.assertEqual(self.portal.portal_types.getTypeInfo('File'), attachments[0].getTypeInfo())

        """
        from Products.PloneArticle.ArticleCore import ArticleContents
        self.failUnless(isinstance(attachments[0], ArticleContents),
                        'Value is %s' % type(attachments[0]))
        """
        #test content
        self.assertEqual('file_contents', str(attachments[0]))



    def testGetWorkingdoc(self, ):
        """
        """
        self.testAddWorkingdoc()
        contents = self.meeting1.meeting_attachments
        attachments = contents[1].getContents(self.meeting1)

        #attachment uid is non existant
        uid = 'No chance it exists'
        self.assertEqual(self.meeting1.getWorkingdoc(uid), None)

        #get the uid of added attachment
        uid = attachments[0].UID()
        #get attachment by uid
        attachment = self.meeting1.getWorkingdoc(uid)
        self.assertEqual(self.meeting1.getWorkingdoc(uid), attachments[0])
        #the return attachment is an File
        self.assertEqual(self.portal.portal_types.getTypeInfo('File'), attachment.getTypeInfo())


    def testAddInformationdoc(self, ):
        """
        """
        #create meeting1 and add attachment
        self.addInformationdoc()
        contents = self.meeting1.meeting_attachments
        attachments = contents[2].getContents(self.meeting1) #workingdoc has an index of 1 
        #the meeting1 has only one attachment
        self.assertEqual(1, len(attachments))

        #the attachment added is an File
        self.assertEqual(self.portal.portal_types.getTypeInfo('File'), attachments[0].getTypeInfo())

        """
        from Products.PloneArticle.ArticleCore import ArticleContents
        self.failUnless(isinstance(attachments[0], ArticleContents),
                        'Value is %s' % type(attachments[0]))
        """
        #test content
        self.assertEqual('file_contents', str(attachments[0]))

    def testGetInformationdoc(self, ):
        """
        """
        self.testAddInformationdoc()
        contents = self.meeting1.meeting_attachments
        attachments = contents[2].getContents(self.meeting1)

        #attachment uid is non existant
        uid = 'No chance it exists'
        self.assertEqual(self.meeting1.getInformationdoc(uid), None)

        #get the uid of added attachment
        uid = attachments[0].UID()
        #get attachment by uid
        attachment = self.meeting1.getInformationdoc(uid)
        self.assertEqual(self.meeting1.getInformationdoc(uid), attachments[0])
        #the return attachment is an File
        self.assertEqual(self.portal.portal_types.getTypeInfo('File'), attachment.getTypeInfo())


    def testAddReferencedoc(self, ):
        """
        """
        #create meeting1 and add attachment
        self.addReferencedoc()
        contents = self.meeting1.meeting_attachments
        attachments = contents[3].getContents(self.meeting1) #workingdoc has an index of 1 
        #the meeting has only one referencedoc
        self.assertEqual(1, len(attachments))

        #the attachment added is an File
        self.assertEqual(self.portal.portal_types.getTypeInfo('File'), attachments[0].getTypeInfo())

        """
        from Products.PloneArticle.ArticleCore import ArticleContents
        self.failUnless(isinstance(attachments[0], ArticleContents),
                        'Value is %s' % type(attachments[0]))
        """
        #test content
        self.assertEqual('file_contents', str(attachments[0]))


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestContentCreation))
    return suite
