#
# ReferencesManagerWidget by David Bain <dbain@alteroo.com>
#

from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from AccessControl import ClassSecurityInfo



# The widget itself

class ReferencesManagerWidget(TypesWidget):
    """This widget adds support for uploading references into documents. To 
    support this, you must use it on a folderish type (derived from BaseFolder) 
    with 'FileReference' in the allowed_content_types. Create a BooleanField 
    and use this widget. This will display a form at the bottom of your edit 
    form (presuming it's the last widget, which it probably ought to be) where 
    you can upload images into your content type. The boolean field itself is
    used to select whether an reference download box should be presented. This
    is similar to the "related items" box in Plone.
    
    Content editors may also reference the images directly in their body
    text.
    
    Caveats: In the edit macro, the upload button may steal the default 
    enter-key-press in base_edit.
    """

    # Use the base class properties, and add two of our own
    _properties = TypesWidget._properties.copy()
    _properties.update({'macro'     : 'widget_referencesmanager',
                        'expanded'  : False,
        'size' : '',
        'helper_js': ('referencebrowser.js',),
        'default_search_index':'SearchableText',
        'show_indexes':0,
        'available_indexes':{},
        'allow_search':1,
        'allow_browse':1,
        'startup_directory':'',
        'base_query':'',
        'force_close_on_insert':0,
        'search_catalog':'portal_catalog',
                        },)


    security=ClassSecurityInfo()
    security.declarePublic('getBaseQuery')
    def getBaseQuery(self, instance, field):
        """Return base query to use for content search"""
        query = self.base_query
        if query:
            if type(query) is StringType and shasattr(instance, query):
                method = getattr(instance, query)
                results = method()
            elif callable(query):
                results = query()
            elif isinstance(query,dict):
                results = query
        else:
            results = {}

        # Add portal type restrictions based on settings in field, if not part
        # of original base_query the template tries to do this, but ignores
        # allowed_types_method, which should override allowed_types
        if not results.has_key('portal_type'):
            allowed_types = getattr(field, 'allowed_types', ())
            allow_method = getattr(field, 'allowed_types_method', None)
            if allow_method is not None:
                meth = getattr(instance, allow_method)
                allowed_types = meth()
            results['portal_type']=allowed_types

        return results

# Register the widget with Archetypes
registerWidget(ReferencesManagerWidget,
               title = 'References manager widget',
               description= ('Renders controls for uploading references to documents',),
               used_for = ('Products.Archetypes.Field.BooleanField',)
               )
